
###############################################################################
#
# RSSBlog::Publish::Avantgo.pm 0.1
# Aaron Straup Cope <aaron@aaronland.net>
#
# Based on (to put it lightly) Jonathan Eisenzopf's Weblog
# http://webreference.com/perl/tutorial/17
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
###############################################################################

# $Id: Avantgo.pm,v 1.10 2001/02/01 03:19:38 asc Exp $

package RSSBlog::Publish::Avantgo;
use strict;

use File::Basename;

sub init {
    my $self = shift;
    return 1;
}

sub build_avantgo {
    my $self = shift;
    my $args = { @_ };

    my $rss  = $args->{'rss'};

    my $type           = 'avantgo';
    my $chan_url       = $type."_url";
    my $chan_title_max = $type."_title_max";
    my $chan_story     = $type."_story";
    my $chan_desc_max  = $type."_desc_max";
    
    # exit the subroutine if the the <avantgo> element does not exist or
    # does not contain a value for the channel in the config.xml file.
    if (! defined($self->channel_data($type))) { $self->last_error("Not defined!"); return 0; }
    
    # build the name of the template using the <tmpl_dir> element and
    # html filename as specified in the config.xml file.
    my $tmpl = join("/",$self->channel_data("tmpl_dir"),$self->config_data($type));

    # create a new instance of the HTML::Template module using the 
    # HTML template for the channel. Each channel has its own template
    # directory with one or more templates for each output format.
    # We turn off die_on_bad_params so that the program won't croak
    # if users remove one of the template parameters from the template
    # file.
    my $template = HTML::Template->new(filename          => $tmpl,
				       die_on_bad_params => 0);

    # open the HTML output file for write or croak
    my $htmlout = $self->channel_data($type);

    my $dirname = &dirname($htmlout);
    if (! -d $dirname) { $self->make_dir($dirname,0751) || return 0; }

    open(HTML,">$htmlout") || &{ $self->last_error(" Cannot write to $htmlout : $!"); return 0; };
    
    # create the array that will hold the RSS items info.
    my @index_array = ();
    
    # build the data structure for index file
    my $count = 1;
    
    # loop over each RSS item and create the output file.
    foreach my $item (@{$rss->{'items'}}) {

	# reject the item unless it contains a title and link
	next unless $item->{'title'} && $item->{'link'};

	# exit the loop if we've reach the maximum number of items
	# to output per the avantgo setting in the <max_items> element
	# of the config.xml file.
	last unless $count <= $self->channel_data("max_items",$type);
	
	# build a hash for the item values
	my %row = ();

	# build the link to the RSS item using the <avantgo_url> element
	# in the config.xml file
	$row{'LINK'} = join("/",($self->channel_data($chan_url),"$count.html"));

	# concatenate the RSS title to the length specified in the
	# <avantgo_title_max> element of the config.xml file
	$row{'TITLE'} = $self->concat_text($self->channel_data($chan_title_max),$item->{'title'});
	
	# push the rss item values onto the array, which will be used to
	# build the index.html file
	push(@index_array,\%row);

	# build story file for the rss item based on the template
	# directory and filename per the config.xml file.
	my $story_tmpl = join("/",$self->channel_data("tmpl_dir"),$self->config_data("templates",$chan_story));

	# create a new instance of the HTML::Template module using the
	# story template
	my $story = HTML::Template->new(filename          => $story_tmpl,
					die_on_bad_params => 0);
	
	# open the output file or croak
	my $storyout = join("/",$self->channel_data($chan_story),"$count.html");
	open(STORY,">$storyout") || &{ $self->last_error("Cannot write to $storyout"); return 0; };
	
	# set the story template values based on the rss item
	$story->param(STORY_TITLE => $item->{'title'});
	$story->param(DESCRIPTION => $self->concat_text($self->channel_data($chan_desc_max),$item->{'description'}));
	$story->param(LINK        => $item->{'link'});
	
	# print the story file
	print STORY $story->output();
	close(STORY);
	
	# increment story counter
	$count++;
    }
    
    # set item array in template
    $template->param(RSS_LOOP => \@index_array);
    
    # set title
    $template->param(FULLNAME => $rss->channel('title'));
    
    # generate the HTML index file
    print HTML $template->output();
    close(HTML);
    
    # Avantgo build was successful
    return 1;
}

return 1;
