
###############################################################################
#
# RSSBlog::Publish::Voicexml.pm 0.1
# Aaron Straup Cope <aaron@aaronland.net>
#
# Based on (to put it lightly) Jonathan Eisenzopf's Weblog
# http://webreference.com/perl/tutorial/17
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
###############################################################################

# $Id: Voicexml.pm,v 1.7 2001/02/01 03:20:58 asc Exp $

package RSSBlog::Publish::Voicexml;
use strict;

use File::Basename;

sub init {
    my $self = shift;
    return 1;
}

sub build_voicexml {
    my $self = shift;

    my $args = { @_ };
    my $rss  = $args->{'rss'};
    
    # exit if no value has been defined for the <vxml> element in the
    # config.xml file for the channel
    if (! defined($self->channel_data("vxml"))) { $self->last_error("No Voice XML channel."); return 0; }
    
    # build the name of the template using the <tmpl_dir> element and
    # html filename as specified in the config.xml file.
    my $tmpl = join("/",$self->channel_data("tmpl_dir"),$self->channel_data("templates","vxml"));

    # create a new instance of the HTML::Template module using the 
    # HTML template for the channel. Each channel has its own template
    # directory with one or more templates for each output format.
    # We turn off die_on_bad_params so that the program won't croak
    # if users remove one of the template parameters from the template
    # file.
    my $template = HTML::Template->new(filename          => $tmpl,
				       die_on_bad_params => 0);
    
    # open the VXML output file for write or croak
    my $vxmlout = $self->channel_data("vxml");

    my $dirname = &dirname($vxmlout);
    if (! -d $dirname) { $self->make_dir($dirname,0751) || return 0; }

    open(HTML,">$vxmlout") || &{ $self->last_error(" Can not write to $vxmlout : $!"); return 0; };
    
    # We need to build an array that will be populated by RSS
    # item values and outputted via the HTML::Template template
    # file
    my @tmpl_array = ();
    
    # build the data structure for HTML::Template
    my $count = 1;

    # loop over each RSS item
    foreach my $item (@{$rss->{'items'}}) {
	# skip the item unless there's a title and link
	next unless $item->{'title'} && $item->{'link'};

	# exit the loop if we've reached the item limit specifed
	# in the <max_items> element in the config.xml file
	last unless $count <= $self->channel_data("max_items","vxml");

	# now we build the has that contains the item contents and
	# push it the the array
	my %row = ();
	$row{'TITLE'} = $item->{'title'};
	$row{'DESCRIPTION'} = $item->{'description'};
	push(@tmpl_array,\%row);
	$count++;
    }
    
    # set item array in template
    $template->param(RSS_LOOP => \@tmpl_array);
    
    # set the title in the template
    $template->param(FULLNAME    => $rss->channel('title'));
    $template->param(DESCRIPTION => $rss->channel('description'));
    
    # generate the HTML file
    print HTML $template->output();
    close(HTML);

    # VoiceXML file was written successfully
    return 1;
}

return 1;
