#!/yer/perl/here -w

###############################################################################
#
# rssblog.pl 0.1
# Aaron Straup Cope <aaron@aaronland.net>
#
# Based on (to put it lightly) Jonathan Eisenzopf's Weblog
# http://webreference.com/perl/tutorial/17
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
###############################################################################

use strict;
use lib qw ( /yer/lib/here );

# Modules 
use RSSBlog;

# Modules CPAN
use CGI::Pretty;
use CGI::Carp qw(fatalsToBrowser);
use Data::Dumper;

# Objects
my $blog;
my $cgi;

my $config_file  = "/yer/config/here.xml";

{
    &main;
    exit;
}

sub main {

    $cgi  = new CGI::Pretty || die; 
    $blog = new RSSBlog(config=>$config_file) || &fatal("Failed to create RSSBlog!");

    my $current_channel = $cgi->param('channel');
    my $current_rss     = $cgi->param('rss');
    
    # Who do I want to be
    if (! defined($current_channel)) {
	$current_channel = $blog->config_data("default_channel") || &fatal("No channel.");
    }

    $blog->channel($current_channel);
    
    # Do we even have an RSS to read?
    $current_rss = $blog->channel_data($current_rss) || $blog->channel_data("rss1.0");

    # Who am I?
    (defined ($current_rss)) ? $blog->rss($current_rss) : &fatal("No RSS defined.");
    
    my $rss;
    my $move;

    for ($cgi->param) { if ($_ =~ /ITEM/) { $move++; } }

    # user selected "Build RSS" button
    if ( ($cgi->param('ACTION') eq "Build RSS") || ($move) ){ 
	my $data = $cgi->Vars;

	$rss  = $blog->generate_rss(data=>$data,move=>$move)
	    || &fatal("Ack! Failed to generate RSS : ".$blog->last_error);

	$blog->publish(rss=>$rss,formats=>["html","palm","avantgo","vxml","wml"]) 
	    || &fatal("Ack! Failed to publish : ".$blog->last_error);
    } 

    else { 
	$rss = $blog->parse_rss($blog->rss);
    }

    &print_form($rss);
    return 1;
}

sub print_form {
    my $rss = shift;

    # Make sure we have a valid RSS object
    &fatal("Not a valid RSS object.") unless (ref($rss) eq "XML::RSS");

    # set channel params
    $cgi->param(-name=>'ctitle',     -value=>$rss->channel('title'));
    $cgi->param(-name=>'cdesc',      -value=>$rss->channel('description'));
    $cgi->param(-name=>'clink',      -value=>$rss->channel('link'));
    $cgi->param(-name=>'cpublisher', -value=>$rss->{channel}->{dc}->{publisher});
    $cgi->param(-name=>'ccreator',   -value=>$rss->{channel}->{dc}->{creator});
    $cgi->param(-name=>'crights',    -value=>$rss->{channel}->{dc}->{rights});
    $cgi->param(-name=>'cdate',      -value=>$rss->{channel}->{dc}->{date});
    $cgi->param(-name=>'csubject',   -value=>$rss->{channel}->{dc}->{subject});
    
    # set image params
    $cgi->param(-name=>'ititle',  -value=>$rss->image('title'));
    $cgi->param(-name=>'iurl',    -value=>$rss->image('url'));
    $cgi->param(-name=>'ilink',   -value=>$rss->image('link'));
    $cgi->param(-name=>'iwidth',  -value=>$rss->image('width'));
    $cgi->param(-name=>'iheight', -value=>$rss->image('height'));
    $cgi->param(-name=>'idesc',   -value=>$rss->image('description'));

    # set text input params
    $cgi->param(-name=>'tititle', -value=>$rss->textinput('title'));
    $cgi->param(-name=>'tidesc',  -value=>$rss->textinput('description'));
    $cgi->param(-name=>'tiname',  -value=>$rss->textinput('name'));
    $cgi->param(-name=>'tilink',  -value=>$rss->textinput('link'));


    # set item params
    for (my $i=0; $i < @{$rss->{'items'}}; $i++) {
        my $item = $rss->{'items'}->[$i];
        $cgi->param(-name=>"item".($i+1)."_title", -value=>$item->{'title'});
        $cgi->param(-name=>"item".($i+1)."_link", -value=>$item->{'link'});
	$cgi->param(-name=>"item".($i+1)."_desc", -value=>$item->{'description'});
	$cgi->param(-name=>"item".($i+1)."_creator", -value=>$item->{dc}->{'creator'});
	$cgi->param(-name=>"item".($i+1)."_subject", -value=>$item->{dc}->{'subject'});
	$cgi->param(-name=>"item".($i+1)."_date", -value=>$item->{dc}->{'date'});
    }

    my $ctitle     = $cgi->param('ctitle');
    my $cdesc      = $cgi->param('cdesc');
    my $clink      = $cgi->param('clink');
    my $cpublisher = $cgi->param('cpublisher');
    my $ccreator   = $cgi->param('ccreator');
    my $crights    = $cgi->param('crights');
    my $csubject   = $cgi->param('csubject');
    my $ititle     = $cgi->param('ititle');
    my $iurl       = $cgi->param('iurl');
    my $ilink      = $cgi->param('ilink');
    my $iwidth     = $cgi->param('iwidth');
    my $iheight    = $cgi->param('iheight');
    my $idesc      = $cgi->param('idesc');
    my $tititle    = $cgi->param('tititle');
    my $tidesc     = $cgi->param('tidesc');
    my $tiname     = $cgi->param('tiname');
    my $tilink     = $cgi->param('tilink');
    my $rss_url    = $cgi->param('rss_url');

    # Colors
    my $green = '#80FF80';
    my $blue  = '#8080FF';
    my $grey  = '#CCCCCC';

    print 
	$cgi->header,
	$cgi->start_html(-title => 'Weblog Manager', -bgcolor => '#ffffff'),
	#$cgi->pre(&Dumper($rss)),
	#$cgi->pre(&Dumper($cgi->Vars)),
	$cgi->h1({-align=>'center'},'Weblog Manager'),
	$cgi->startform(-method=>"post",-action=>$cgi->url),
	$cgi->hidden(-name=>'channel', -value=>$blog->channel,-override=>1);

    # Legend
    my $legend = "<font color=$green>New Item</font> | ";
    $legend   .= "<font color=$grey>  Existing Item</font> | ";
    $legend   .= "<font color=$blue> Channel Info </font>\n"; 

    print $cgi->p({-align=>"center"},$cgi->strong("Legend : $legend"));
    
    # Super Table 
    print table({-border=>1, -width=>'100%',-align=>'center', -cellspacing=>1, -cellpadding=>3});
    # New Item Table
    print table({-border=>1, -width=>'600', -align=>'center', -cellspacing=>1, -cellpadding=>3,bgcolor=>$green});
    
    # new channel item
    print 
	$cgi->Tr({-align=>'left',-valign=>'middle'},
		   $cgi->th({-colspan=>3,-bgcolor=>'#000000'}, 
			    $cgi->span({-class=>"title"},"New Item "))),
	$cgi->Tr({-align=>'left',-valign=>'middle'},
		 $cgi->td($cgi->strong("Title")),
		 $cgi->td($cgi->textfield(-name=>'newtitle1',
					  -size=>50,
					  -default=>'',
					  -override=>1,
					  ),
			  $cgi->submit(-name=>'action',-value=>'Build RSS'))),
	$cgi->Tr({-align=>'left',-valign=>'middle'},
		 $cgi->td($cgi->strong("Link")),
		 $cgi->td($cgi->textfield(-name=>'newlink1',
					  -size=>50,
					  -default=>'',
					  -override=>1,
					  ))),
	$cgi->Tr({-align=>'left',-valign=>'middle'},
		 $cgi->td($cgi->strong("Desc")),
		 $cgi->td($cgi->textarea(-name=>'newdesc1',			      
					 -columns=>50,
					 -rows=>5,
					 -default=>'',
					 -override=>1,
					 ))),
	$cgi->Tr({-align=>'left',-valign=>'middle'},
		 $cgi->td($cgi->strong("Creator:")),
		 $cgi->td($cgi->textfield(-name=>'newcreator1',
					  -size=>50,
					  -default=>'',
					  -override=>1,
					  ))),
	$cgi->Tr({-align=>'left',-valign=>'middle'},
		 $cgi->td($cgi->strong("Subject:")),
		 $cgi->td($cgi->textfield(-name=>'newsubject1',
					  -size=>50,
					  -default=>'',
					  -override=>1,
					  )));
    
    # Creates the space between the new and the old items

    print 
	"</table>\n",
	"</table>\n",
	$cgi->p("&nbsp;");

    # End of new item table
    
    # Re-instantiate Super Table
    print $cgi->table({-border=>1, -width=>'100%', -align=>'center', -cellspacing=>1, -cellpadding=>3});
    print $cgi->table({-border=>1, -width=>'600', -align=>'center', -cellspacing=>1, -cellpadding=>3, bgcolor=>$grey});
 
    # channel items
    for (my $i = 1; $i <= 15; $i++) {
        my %title;
        my %link;
	my %desc;
	my %creator;
	my %subject;
	my %date;

        $title{'name'}    = "item".$i."_title";
        $title{'value'}   = $cgi->param($title{'name'});
	$desc{'name'}     = "item".$i."_desc";
	$desc{'value'}    = $cgi->param($desc{'name'});
        $link{'name'}     = "item".$i."_link";
        $link{'value'}    = $cgi->param($link{'name'});
	$creator{'name'}  = "item".$i."_creator";
	$creator{'value'} = $cgi->param($creator{'name'});
	$subject{'name'}  = "item".$i."_subject";
	$subject{'value'} = $cgi->param($subject{'name'});
	$date{'name'}     = "item".$i."_date";
	$date{'value'}    = $cgi->param($date{'name'});

	my $ucode;

	if (($i != 1) &&  ($i != 15)) {
	    $ucode = $cgi->table({-align=>'left'},
				 $cgi->Tr($cgi->td({-rowspan=>2},
						   $cgi->submit(-name=>"ITEM$i" , 
								-value=>" up    "), 
						   $cgi->br,  
						   $cgi->submit(-name=>"ITEM$i", 
								-value=>"down"),
						   ),
					  ));
	} else {
	    $ucode = undef;
	}
	
        print $cgi->Tr({-align=>'left',-valign=>'middle'},
                 [
                  $cgi->th({-colspan=>3,-bgcolor=>'#000000'},
			   $cgi->span({-class=>"title"},"Item$i")),
                  $cgi->td({-align=>'right'},
			   [$cgi->strong("Title"),
			    $cgi->textfield(-name=>$title{'name'},
					    -default=>$title{'value'},
					    -size=>50,
					    -override=>1,),
			    $cgi->submit(-name=>'ACTION',-value=>'Build RSS')
			    ]),
                  $cgi->td({-align=>'right'},
			   [$cgi->strong("Link"),
			    $cgi->textfield(-name=>$link{'name'},
					    -default=>$link{'value'},
					    -size=>50,
					    -override=>1,),
			    $cgi->checkbox(-name=> "delete" . $i, -label => ' Delete')
			    ]),
		  $cgi->td({-align=>'right'},
			   [$cgi->strong("Desc"),
			    $cgi->textarea(-name=>$desc{'name'},
					   -default=>$desc{'value'},
					   -columns=>50,
					   -rows=>5,
					   -override=>1,),
			    $ucode,
			    ]),
		  $cgi->td({-align=>'right'},
			   [$cgi->strong('Creator:'),
			    $cgi->textfield(-name=>$creator{'name'},
					    -size=>50,
					    -default=>$creator{'value'},
					    -override=>1,),
			    ]),
	          $cgi->td({-align=>'right'},
			   [$cgi->strong('Subject:'),
			    $cgi->textfield(-name=>$subject{'name'},
					    -size=>50,
					    -default=>$subject{'value'},
					    -override=>1,),
			    ]),
		  $cgi->td({-align=>'right'},
			   [$cgi->strong('Date:'),
			    $cgi->textfield(-name=>$date{'name'},
					    -size=>50,
					    -default=>$date{'value'},
					    -override=>1,
					    ),
			    ]),
		  ]);
    }

    print "</table>\n";
    print "</table>\n";
    print $cgi->p("&nbsp;");
    
    # Summary Section
    print $cgi->table({-border=>1, -width=>'100%',-align=>'center', -cellspacing=>1, -cellpadding=>3});
    print $cgi->table({-border=>1, -width=>'600', -align=>'center', -cellspacing=>1, -cellpadding=>3,bgcolor=>$blue});

    # channel summary
    print $cgi->Tr({-align=>'left',-valign=>'middle'},
		   [
		    $cgi->th({-colspan=>2,-BGCOLOR=>'#000000'},
			     $cgi->span({-class=>"title"},'Channel Summary')),
		    $cgi->td({-align=>'right'},
			     [$cgi->strong('Channel Title'),
			      $cgi->textfield(-name=>'ctitle',
					      -default=>$ctitle,
					      -size=>50,
					      -override=>1,
					      ),
			      ]),
		    $cgi->td({-align=>'right'},
			     [$cgi->strong('Channel Link'),
			      $cgi->textfield(-name=>'clink',
					      -default=>$clink,
					      -size=>50,
					      -override=>1,
					      ),
			      ]),
		    $cgi->td({-align=>'right'},
			     [$cgi->strong('Description'),
			      $cgi->textarea(-name=>'cdesc',
					     -default=>$cdesc,
					     -rows=>5,
					     -columns=>50,
					     -override=>1,
					     ),
			      ]),
		    $cgi->td({-align=>'right'},
			     [$cgi->strong('Publisher'),
			      $cgi->textfield(-name=>'cpublisher',
					      -default=>$cpublisher,
					      -size=>50,
					      -override=>1,
					      ),
			      ]),
		    $cgi->td({-align=>'right'},
			     [$cgi->strong('Editor'),
			      $cgi->textfield(-name=>'ccreator',
					      -default=>$ccreator,
					      -size=>50,
					      -override=>1,
					      ),
			      ]),
		    $cgi->td({-align=>'right'},
			     [$cgi->strong('Copyright'),
			      $cgi->textfield(-name=>'crights',
					      -default=>$crights,
					      -size=>50,
					      -override=>1,
					      ),
			      ]),
		    $cgi->td({-align=>'right'},
			     [$cgi->strong('Subject'),
			      $cgi->textfield(-name=>'csubject',
					      -default=>$csubject,
					      -size=>50,
					      -override=>1,
					      ),
			      ]),
		    ]);

    # channel image
    print $cgi->Tr({-align=>'left',-valign=>'middle'},
             [
              $cgi->th({-colspan=>2,-bgcolor=>'#000000'},
                 span({-class=>"title"},'Channel Image')
                 ),
              $cgi->td({-align=>'right'},
		       [$cgi->strong('Image Title'),
			$cgi->textfield(-name=>'ititle',
					-default=>$ititle,
					-size=>50,
					-override=>1,
					),
			]),
              $cgi->td({-align=>'right'},
		       [$cgi->strong('Image URL'),
			$cgi->textfield(-name=>'iurl',
					-default=>$iurl,
					-size=>50,
					-override=>1,
					),
			]),
              $cgi->td({-align=>'right'},
		       [$cgi->strong('Image Link'),
			$cgi->textfield(-name=>'ilink',
					-default=>$ilink,
					-size=>50,
					-override=>1,
					),
			]),
	      $cgi->td({-align=>'right'},
		       [$cgi->strong('Image Width'),
			$cgi->textfield(-name=>'iwidth',
					-default=>$iwidth,
					-size=>50,
					-override=>1,
					),
			]),
	      $cgi->td({-align=>'right'},
		       [$cgi->strong('Image Height'),
			$cgi->textfield(-name=>'iheight',
					-default=>$iheight,
					-size=>50,
					-override=>1,
					),
			]),
	      $cgi->td({-align=>'right'},
		       [$cgi->strong('Image Desc'),
			$cgi->textarea(-name=>'idesc',
				       -default=>$idesc,
				       -columns=>50,
				       -rows=>5,
				       -override=>1,
				       ),
			]),
              ]);

    # channel textarea
    print Tr({-align=>'LEFT',-valign=>'MIDDLE'},
             [
              $cgi->th({-colspan=>2,-bgcolor=>'#000000'},
		       $cgi->span({-class=>"title"},'Text Input')),
              $cgi->td({-align=>'right'},
		       [$cgi->strong('Input Title'),
			$cgi->textfield(-name=>'tititle',
					-default=>$tititle,
					-size=>50,
					-override=>1,
					),
			]),
              $cgi->td({-align=>'right'},
		       [$cgi->strong('Input Name'),
			$cgi->textfield(-name=>'tiname',
					-default=>$tiname,
					-size=>50,
					-override=>1,
					),
			]),
              $cgi->td({-align=>'right'},
		       [$cgi->strong('Input Link'),
			$cgi->textfield(-name=>'tilink',
					-default=>$tilink,
					-size=>50,
					-override=>1,
					),
			]),
              $cgi->td({-align=>'right'},
		       [$cgi->strong('Input Desc'),
			textarea(-name=>'tidesc',
				 -default=>$tidesc,
				 -columns=>50,
				 -rows=>5,
				 -override=>1,
				 ),
			]),
              ]);

    print 
	"</table>\n",
	"</table>\n",
	$cgi->end_form;

    return 1;
}

sub fatal {
    my $msg = shift;
    
    print 
	$cgi->header, 
	$cgi->start_html,
	$cgi->strong("Error!"),$cgi->hr,
	$cgi->p($msg),
	$cgi->end_html;
    exit;
}
